@@ -39,7 +39,15 @@ THE SOFTWARE.  */
 #define FT_STDIN (1<<3)
 
 char *bcftools_version(void);
-void error(const char *format, ...) HTS_NORETURN;
+
+/// Report an error and exit -1
+void error(const char *format, ...) HTS_NORETURN HTS_FORMAT(HTS_PRINTF_FMT, 1, 2);
+
+/// Report an error and exit -1.  If errno != 0, appends strerror(errno).
+//  Note: unlike error() above, the message should not end with "\n" as a
+//  newline will be added by the function.
+void error_errno(const char *format, ...) HTS_NORETURN HTS_FORMAT(HTS_PRINTF_FMT, 1, 2);
+
 void bcf_hdr_append_version(bcf_hdr_t *hdr, int argc, char **argv, const char *cmd);
 const char *hts_bcf_wmode(int file_type);
 
