@@ -36,6 +36,7 @@
 #include <htslib/kstring.h>
 #include <htslib/synced_bcf_reader.h>
 #include <htslib/kseq.h>
+#include <htslib/bgzf.h>
 #include "regidx.h"
 #include "bcftools.h"
 #include "rbuf.h"
@@ -49,6 +50,7 @@
 #define PICK_ALT   2
 #define PICK_LONG  4
 #define PICK_SHORT 8
+#define PICK_IUPAC 16
 
 typedef struct
 {
@@ -73,11 +75,14 @@ typedef struct
     int fa_length;      // region's length in the original sequence (in case end_pos not provided in the FASTA header)
     int fa_case;        // output upper case or lower case?
     int fa_src_pos;     // last genomic coordinate read from the input fasta (0-based)
+    char prev_base;     // this is only to validate the REF allele in the VCF - the modified fa_buf cannot be used for inserts following deletions, see 600#issuecomment-383186778
+    int prev_base_pos;  // the position of prev_base
+    int prev_is_insert;
 
     rbuf_t vcf_rbuf;
     bcf1_t **vcf_buf;
     int nvcf_buf, rid;
-    char *chr;
+    char *chr, *chr_prefix;
 
     regidx_t *mask;
     regitr_t *itr;
@@ -95,8 +100,8 @@ typedef struct
     FILE *fp_out;
     FILE *fp_chain;
     char **argv;
-    int argc, output_iupac, haplotype, allele, isample;
-    char *fname, *ref_fname, *sample, *output_fname, *mask_fname, *chain_fname;
+    int argc, output_iupac, haplotype, allele, isample, napplied;
+    char *fname, *ref_fname, *sample, *output_fname, *mask_fname, *chain_fname, missing_allele;
 }
 args_t;
 
@@ -204,7 +209,7 @@ static void init_data(args_t *args)
 {
     args->files = bcf_sr_init();
     args->files->require_index = 1;
-    if ( !bcf_sr_add_reader(args->files,args->fname) ) error("Failed to open %s: %s\n", args->fname, bcf_sr_strerror(args->files->errnum));
+    if ( !bcf_sr_add_reader(args->files,args->fname) ) error("Failed to read from %s: %s\n", !strcmp("-",args->fname)?"standard input":args->fname, bcf_sr_strerror(args->files->errnum));
     args->hdr = args->files->readers[0].header;
     args->isample = -1;
     if ( args->sample )
@@ -237,7 +242,7 @@ static void init_data(args_t *args)
         if ( ! args->fp_out ) error("Failed to create %s: %s\n", args->output_fname, strerror(errno));
     }
     else args->fp_out = stdout;
-    if ( args->isample<0 ) fprintf(stderr,"Note: the --sample option not given, applying all records\n");
+    if ( args->isample<0 ) fprintf(stderr,"Note: the --sample option not given, applying all records regardless of the genotype\n");
     if ( args->filter_str )
         args->filter = filter_init(args->hdr, args->filter_str);
 }
@@ -264,7 +269,7 @@ static void init_region(args_t *args, char *line)
     char *ss, *se = line;
     while ( *se && !isspace(*se) && *se!=':' ) se++;
     int from = 0, to = 0;
-    char tmp, *tmp_ptr = NULL;
+    char tmp = 0, *tmp_ptr = NULL;
     if ( *se )
     {
         tmp = *se; *se = 0; tmp_ptr = se;
@@ -284,7 +289,8 @@ static void init_region(args_t *args, char *line)
     args->chr = strdup(line);
     args->rid = bcf_hdr_name2id(args->hdr,line);
     if ( args->rid<0 ) fprintf(stderr,"Warning: Sequence \"%s\" not in %s\n", line,args->fname);
-    args->fa_buf.l = 0;
+    args->prev_base_pos = -1;
+    args->fa_buf.l  = 0;
     args->fa_length = 0;
     args->fa_end_pos = to;
     args->fa_ori_pos = from;
@@ -295,7 +301,7 @@ static void init_region(args_t *args, char *line)
     args->vcf_rbuf.n = 0;
     bcf_sr_seek(args->files,line,args->fa_ori_pos);
     if ( tmp_ptr ) *tmp_ptr = tmp;
-    fprintf(args->fp_out,">%s\n",line);
+    fprintf(args->fp_out,">%s%s\n",args->chr_prefix?args->chr_prefix:"",line);
     if (args->chain_fname )
     {
         args->chain = init_chain(args->chain, args->fa_ori_pos);
@@ -327,7 +333,7 @@ static void unread_vcf_line(args_t *args, bcf1_t **rec_ptr)
 {
     bcf1_t *rec = *rec_ptr;
     if ( args->vcf_rbuf.n >= args->vcf_rbuf.m )
-        error("FIXME: too many overlapping records near %s:%d\n", bcf_seqname(args->hdr,rec),rec->pos+1);
+        error("FIXME: too many overlapping records near %s:%"PRId64"\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
 
     // Insert the new record in the buffer. The line would be overwritten in
     // the next bcf_sr_next_line call, therefore we need to swap it with an
@@ -371,13 +377,10 @@ static void flush_fa_buffer(args_t *args, int len)
 }
 static void apply_variant(args_t *args, bcf1_t *rec)
 {
-    if ( rec->n_allele==1 ) return;
+    static int warned_haplotype = 0;
+
+    if ( rec->n_allele==1 && !args->missing_allele ) return;
 
-    if ( rec->pos <= args->fa_frz_pos )
-    {
-        fprintf(stderr,"The site %s:%d overlaps with another variant, skipping...\n", bcf_seqname(args->hdr,rec),rec->pos+1);
-        return;
-    }
     if ( args->mask )
     {
         char *chr = (char*)bcf_hdr_id2name(args->hdr,args->rid);
@@ -394,37 +397,84 @@ static void apply_variant(args_t *args, bcf1_t *rec)
         if ( !fmt ) return;
 
         if ( fmt->type!=BCF_BT_INT8 )
-            error("Todo: GT field represented with BCF_BT_INT8, too many alleles at %s:%d?\n",bcf_seqname(args->hdr,rec),rec->pos+1);
+            error("Todo: GT field represented with BCF_BT_INT8, too many alleles at %s:%"PRId64"?\n",bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
         uint8_t *ptr = fmt->p + fmt->size*args->isample;
 
-        if ( args->haplotype )
+        enum { use_hap, use_iupac, pick_one } action = use_hap;
+        if ( args->allele==PICK_IUPAC )
         {
-            if ( args->haplotype > fmt->n ) error("Can't apply %d-th haplotype at %s:%d\n", args->haplotype,bcf_seqname(args->hdr,rec),rec->pos+1);
-            ialt = ptr[args->haplotype-1];
-            if ( bcf_gt_is_missing(ialt) || ialt==bcf_int32_vector_end ) return;
-            ialt = bcf_gt_allele(ialt);
+            if ( !bcf_gt_is_phased(ptr[0]) && !bcf_gt_is_phased(ptr[fmt->n-1]) ) action = use_iupac;
         }
-        else if ( args->output_iupac ) 
+        else if ( args->output_iupac ) action = use_iupac;
+        else if ( !args->haplotype ) action = pick_one;
+
+        if ( action==use_hap )
+        {
+            if ( args->haplotype > fmt->n )
+            {
+                if ( bcf_gt_is_missing(ptr[fmt->n-1]) || bcf_gt_is_missing(ptr[0]) )
+                {
+                    if ( !args->missing_allele ) return;
+                    ialt = -1;
+                }
+                else 
+                {
+                    if ( !warned_haplotype )
+                    {
+                        fprintf(stderr, "Can't apply %d-th haplotype at %s:%"PRId64". (This warning is printed only once.)\n", args->haplotype,bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
+                        warned_haplotype = 1;
+                    }
+                    return;
+                }
+            }
+            else
+            {
+                ialt = (int8_t)ptr[args->haplotype-1];
+                if ( bcf_gt_is_missing(ialt) || ialt==bcf_int8_vector_end )
+                {
+                    if ( !args->missing_allele ) return;
+                    ialt = -1;
+                }
+                else 
+                    ialt = bcf_gt_allele(ialt);
+            }
+        }
+        else if ( action==use_iupac ) 
         {
             ialt = ptr[0];
-            if ( bcf_gt_is_missing(ialt) || ialt==bcf_int32_vector_end ) return;
-            ialt = bcf_gt_allele(ialt);
+            if ( bcf_gt_is_missing(ialt) || ialt==bcf_int32_vector_end )
+            {
+                if ( !args->missing_allele ) return;
+                ialt = -1;
+            }
+            else
+                ialt = bcf_gt_allele(ialt);
 
             int jalt;
             if ( fmt->n>1 )
             {
                 jalt = ptr[1];
-                if ( bcf_gt_is_missing(jalt) || jalt==bcf_int32_vector_end ) jalt = ialt;
-                else jalt = bcf_gt_allele(jalt);
+                if ( bcf_gt_is_missing(jalt) )
+                {
+                    if ( !args->missing_allele ) return;
+                    ialt = -1;
+                }
+                else if ( jalt==bcf_int32_vector_end ) jalt = ialt;
+                else
+                    jalt = bcf_gt_allele(jalt);
             }
             else jalt = ialt;
-            if ( rec->n_allele <= ialt || rec->n_allele <= jalt ) error("Invalid VCF, too few ALT alleles at %s:%d\n", bcf_seqname(args->hdr,rec),rec->pos+1);
-            if ( ialt!=jalt && !rec->d.allele[ialt][1] && !rec->d.allele[jalt][1] ) // is this a het snp?
+
+            if ( ialt>=0 )
             {
-                char ial = rec->d.allele[ialt][0];
-                char jal = rec->d.allele[jalt][0];
-                if ( !ialt ) ialt = jalt;   // only ialt is used, make sure 0/1 is not ignored
-                rec->d.allele[ialt][0] = gt2iupac(ial,jal);
+                if ( rec->n_allele <= ialt || rec->n_allele <= jalt ) error("Invalid VCF, too few ALT alleles at %s:%"PRId64"\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
+                if ( ialt!=jalt && !rec->d.allele[ialt][1] && !rec->d.allele[jalt][1] ) // is this a het snp?
+                {
+                    char ial = rec->d.allele[ialt][0];
+                    char jal = rec->d.allele[jalt][0];
+                    if ( !ialt ) ialt = jalt;   // only ialt is used, make sure 0/1 is not ignored
+                    rec->d.allele[ialt][0] = gt2iupac(ial,jal);
+                }
             }
         }
         else
@@ -432,8 +482,13 @@ static void apply_variant(args_t *args, bcf1_t *rec)
             int is_hom = 1;
             for (i=0; i<fmt->n; i++)
             {
-                if ( bcf_gt_is_missing(ptr[i]) ) return;  // ignore missing or half-missing genotypes
-                if ( ptr[i]==bcf_int32_vector_end ) break;
+                if ( bcf_gt_is_missing(ptr[i]) )
+                {
+                    if ( !args->missing_allele ) return;  // ignore missing or half-missing genotypes
+                    ialt = -1;
+                    break;
+                }
+                if ( ptr[i]==(uint8_t)bcf_int8_vector_end ) break;
                 ialt = bcf_gt_allele(ptr[i]);
                 if ( i>0 && ialt!=bcf_gt_allele(ptr[i-1]) ) { is_hom = 0; break; }
             }
@@ -442,9 +497,9 @@ static void apply_variant(args_t *args, bcf1_t *rec)
                 int prev_len = 0, jalt;
                 for (i=0; i<fmt->n; i++)
                 {
-                    if ( ptr[i]==bcf_int32_vector_end ) break;
+                    if ( ptr[i]==(uint8_t)bcf_int8_vector_end ) break;
                     jalt = bcf_gt_allele(ptr[i]);
-                    if ( rec->n_allele <= jalt ) error("Broken VCF, too few alts at %s:%d\n", bcf_seqname(args->hdr,rec),rec->pos+1);
+                    if ( rec->n_allele <= jalt ) error("Broken VCF, too few alts at %s:%"PRId64"\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
                     if ( args->allele & (PICK_LONG|PICK_SHORT) )
                     {
                         int len = jalt==0 ? rec->rlen : strlen(rec->d.allele[jalt]);
@@ -466,7 +521,7 @@ static void apply_variant(args_t *args, bcf1_t *rec)
             }
         }
         if ( !ialt ) return;  // ref allele
-        if ( rec->n_allele <= ialt ) error("Broken VCF, too few alts at %s:%d\n", bcf_seqname(args->hdr,rec),rec->pos+1);
+        if ( rec->n_allele <= ialt ) error("Broken VCF, too few alts at %s:%"PRId64"\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
     }
     else if ( args->output_iupac && !rec->d.allele[0][1] && !rec->d.allele[1][1] )
     {
@@ -475,11 +530,41 @@ static void apply_variant(args_t *args, bcf1_t *rec)
         rec->d.allele[1][0] = gt2iupac(ial,jal);
     }
 
+    if ( rec->n_allele==1 && ialt!=-1 ) return; // non-missing reference
+    if ( ialt==-1 )
+    {
+        char alleles[4];
+        alleles[0] = rec->d.allele[0][0];
+        alleles[1] = ',';
+        alleles[2] = args->missing_allele;
+        alleles[3] = 0;
+        bcf_update_alleles_str(args->hdr, rec, alleles);
+        ialt = 1;
+    }
+
+    // Overlapping variant?
+    if ( rec->pos <= args->fa_frz_pos )
+    {
+        // Can be still OK iff this is an insertion (and which does not follow another insertion, see #888).
+        // This still may not be enough for more complicated cases with multiple duplicate positions
+        // and other types in between. In such case let the user normalize the VCF and remove duplicates.
+        int overlap = 0;
+        if ( rec->pos < args->fa_frz_pos || !(bcf_get_variant_type(rec,ialt) & VCF_INDEL) ) overlap = 1;
+        else if ( rec->d.var[ialt].n <= 0 || args->prev_is_insert ) overlap = 1;
+
+        if ( overlap )
+        {
+            fprintf(stderr,"The site %s:%"PRId64" overlaps with another variant, skipping...\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
+            return;
+        }
+        
+    }
+
     int len_diff = 0, alen = 0;
     int idx = rec->pos - args->fa_ori_pos + args->fa_mod_off;
     if ( idx<0 )
     {
-        fprintf(stderr,"Warning: ignoring overlapping variant starting at %s:%d\n", bcf_seqname(args->hdr,rec),rec->pos+1);
+        fprintf(stderr,"Warning: ignoring overlapping variant starting at %s:%"PRId64"\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
         return;
     }
     if ( rec->rlen > args->fa_buf.l - idx )
@@ -489,17 +574,17 @@ static void apply_variant(args_t *args, bcf1_t *rec)
         if ( alen > rec->rlen )
         {
             rec->d.allele[ialt][rec->rlen] = 0;
-            fprintf(stderr,"Warning: trimming variant starting at %s:%d\n", bcf_seqname(args->hdr,rec),rec->pos+1);
+            fprintf(stderr,"Warning: trimming variant starting at %s:%"PRId64"\n", bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
         }
     }
     if ( idx>=args->fa_buf.l ) 
-        error("FIXME: %s:%d .. idx=%d, ori_pos=%d, len=%d, off=%d\n",bcf_seqname(args->hdr,rec),rec->pos+1,idx,args->fa_ori_pos,args->fa_buf.l,args->fa_mod_off);
+        error("FIXME: %s:%"PRId64" .. idx=%d, ori_pos=%d, len=%"PRIu64", off=%d\n",bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1,idx,args->fa_ori_pos,(uint64_t)args->fa_buf.l,args->fa_mod_off);
 
     // sanity check the reference base
     if ( rec->d.allele[ialt][0]=='<' )
     {
         if ( strcasecmp(rec->d.allele[ialt], "<DEL>") )
-            error("Symbolic alleles other than <DEL> are currently not supported: %s at %s:%d\n",rec->d.allele[ialt],bcf_seqname(args->hdr,rec),rec->pos+1);
+            error("Symbolic alleles other than <DEL> are currently not supported: %s at %s:%"PRId64"\n",rec->d.allele[ialt],bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1);
         assert( rec->d.allele[0][1]==0 );           // todo: for now expecting strlen(REF) = 1
         len_diff = 1-rec->rlen;
         rec->d.allele[ialt] = rec->d.allele[0];     // according to VCF spec, REF must precede the event
@@ -507,21 +592,37 @@ static void apply_variant(args_t *args, bcf1_t *rec)
     }
     else if ( strncasecmp(rec->d.allele[0],args->fa_buf.s+idx,rec->rlen) )
     {
-        // fprintf(stderr,"%d .. [%s], idx=%d ori=%d off=%d\n",args->fa_ori_pos,args->fa_buf.s,idx,args->fa_ori_pos,args->fa_mod_off);
-        char tmp = 0;
-        if ( args->fa_buf.l - idx > rec->rlen ) 
-        { 
-            tmp = args->fa_buf.s[idx+rec->rlen];
-            args->fa_buf.s[idx+rec->rlen] = 0;
+        // This is hacky, handle a special case: if SNP or an insert follows a deletion (AAC>A, C>CAA),
+        // the reference base in fa_buf is lost and the check fails. We do not keep a buffer
+        // with the original sequence as it should not be necessary, we should encounter max
+        // one base overlap
+
+        int fail = 1;
+        if ( args->prev_base_pos==rec->pos && toupper(rec->d.allele[0][0])==toupper(args->prev_base) )
+        {
+            if ( rec->rlen==1 ) fail = 0;
+            else if ( !strncasecmp(rec->d.allele[0]+1,args->fa_buf.s+idx+1,rec->rlen-1) ) fail = 0;
         }
-        error(
-            "The fasta sequence does not match the REF allele at %s:%d:\n"
-            "   .vcf: [%s]\n" 
-            "   .vcf: [%s] <- (ALT)\n" 
-            "   .fa:  [%s]%c%s\n",
-            bcf_seqname(args->hdr,rec),rec->pos+1, rec->d.allele[0], rec->d.allele[ialt], args->fa_buf.s+idx, 
-            tmp?tmp:' ',tmp?args->fa_buf.s+idx+rec->rlen+1:""
-            );
+
+        if ( fail )
+        {
+            char tmp = 0;
+            if ( args->fa_buf.l - idx > rec->rlen ) 
+            { 
+                tmp = args->fa_buf.s[idx+rec->rlen];
+                args->fa_buf.s[idx+rec->rlen] = 0;
+            }
+            error(
+                    "The fasta sequence does not match the REF allele at %s:%"PRId64":\n"
+                    "   .vcf: [%s] <- (REF)\n" 
+                    "   .vcf: [%s] <- (ALT)\n" 
+                    "   .fa:  [%s]%c%s\n",
+                    bcf_seqname(args->hdr,rec),(int64_t) rec->pos+1, rec->d.allele[0], rec->d.allele[ialt], args->fa_buf.s+idx,
+                    tmp?tmp:' ',tmp?args->fa_buf.s+idx+rec->rlen+1:""
+                 );
+        }
+        alen = strlen(rec->d.allele[ialt]);
+        len_diff = alen - rec->rlen;
     }
     else
     {
@@ -539,15 +640,31 @@ static void apply_variant(args_t *args, bcf1_t *rec)
         // deletion or same size event
         for (i=0; i<alen; i++)
             args->fa_buf.s[idx+i] = rec->d.allele[ialt][i];
+
         if ( len_diff )
             memmove(args->fa_buf.s+idx+alen,args->fa_buf.s+idx+rec->rlen,args->fa_buf.l-idx-rec->rlen);
+
+        args->prev_base = rec->d.allele[0][rec->rlen - 1];
+        args->prev_base_pos = rec->pos + rec->rlen - 1;
+        args->prev_is_insert = 0;
     }
     else
     {
+        args->prev_is_insert = 1;
+        args->prev_base_pos = rec->pos;
+
         // insertion
         ks_resize(&args->fa_buf, args->fa_buf.l + len_diff);
         memmove(args->fa_buf.s + idx + rec->rlen + len_diff, args->fa_buf.s + idx + rec->rlen, args->fa_buf.l - idx - rec->rlen);
-        for (i=0; i<alen; i++)
+
+        // This can get tricky, make sure the bases unchanged by the insertion do not overwrite preceeding variants.
+        // For example, here we want to get TAA:
+        //      POS REF ALT
+        //      1   C   T
+        //      1   C   CAA
+        int ibeg = 0;
+        while ( ibeg<alen && rec->d.allele[0][ibeg]==rec->d.allele[ialt][ibeg] && rec->pos + ibeg <= args->prev_base_pos  ) ibeg++;
+        for (i=ibeg; i<alen; i++)
             args->fa_buf.s[idx+i] = rec->d.allele[ialt][i];
     }
     if (args->chain && len_diff != 0)
@@ -567,6 +684,7 @@ static void apply_variant(args_t *args, bcf1_t *rec)
     args->fa_buf.l += len_diff;
     args->fa_mod_off += len_diff;
     args->fa_frz_pos  = rec->pos + rec->rlen - 1;
+    args->napplied++;
 }
 
 
@@ -590,10 +708,10 @@ static void mask_region(args_t *args, char *seq, int len)
 
 static void consensus(args_t *args)
 {
-    htsFile *fasta = hts_open(args->ref_fname, "rb");
+    BGZF *fasta = bgzf_open(args->ref_fname, "r");
     if ( !fasta ) error("Error reading %s\n", args->ref_fname);
     kstring_t str = {0,0,0};
-    while ( hts_getline(fasta, KS_SEP_LINE, &str) > 0 )
+    while ( bgzf_getline(fasta, '\n', &str) > 0 )
     {
         if ( str.s[0]=='>' )
         {
@@ -670,8 +788,9 @@ static void consensus(args_t *args)
         destroy_chain(args);
     }
     flush_fa_buffer(args, 0);
-    hts_close(fasta);
+    bgzf_close(fasta);
     free(str.s);
+    fprintf(stderr,"Applied %d variants\n", args->napplied);
 }
 
 static void usage(args_t *args)
@@ -682,23 +801,26 @@ static void usage(args_t *args)
     fprintf(stderr, "       --sample (and, optionally, --haplotype) option will apply genotype\n");
     fprintf(stderr, "       (or haplotype) calls from FORMAT/GT. The program ignores allelic depth\n");
     fprintf(stderr, "       information, such as INFO/AD or FORMAT/AD.\n");
-    fprintf(stderr, "Usage:   bcftools consensus [OPTIONS] <file.vcf>\n");
+    fprintf(stderr, "Usage:   bcftools consensus [OPTIONS] <file.vcf.gz>\n");
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "    -c, --chain <file>         write a chain file for liftover\n");
     fprintf(stderr, "    -e, --exclude <expr>       exclude sites for which the expression is true (see man page for details)\n");
     fprintf(stderr, "    -f, --fasta-ref <file>     reference sequence in fasta format\n");
     fprintf(stderr, "    -H, --haplotype <which>    choose which allele to use from the FORMAT/GT field, note\n");
     fprintf(stderr, "                               the codes are case-insensitive:\n");
-    fprintf(stderr, "                                   1: first allele from GT\n");
-    fprintf(stderr, "                                   2: second allele\n");
+    fprintf(stderr, "                                   1: first allele from GT, regardless of phasing\n");
+    fprintf(stderr, "                                   2: second allele from GT, regardless of phasing\n");
     fprintf(stderr, "                                   R: REF allele in het genotypes\n");
     fprintf(stderr, "                                   A: ALT allele\n");
     fprintf(stderr, "                                   LR,LA: longer allele and REF/ALT if equal length\n");
     fprintf(stderr, "                                   SR,SA: shorter allele and REF/ALT if equal length\n");
+    fprintf(stderr, "                                   1pIu,2pIu: first/second allele for phased and IUPAC code for unphased GTs\n");
     fprintf(stderr, "    -i, --include <expr>       select sites for which the expression is true (see man page for details)\n");
     fprintf(stderr, "    -I, --iupac-codes          output variants in the form of IUPAC ambiguity codes\n");
     fprintf(stderr, "    -m, --mask <file>          replace regions with N\n");
+    fprintf(stderr, "    -M, --missing <char>       output <char> instead of skipping the missing genotypes\n");
     fprintf(stderr, "    -o, --output <file>        write output to a file [standard output]\n");
+    fprintf(stderr, "    -p, --prefix <string>      prefix to add to output sequence names\n");
     fprintf(stderr, "    -s, --sample <name>        apply variants of the given sample\n");
     fprintf(stderr, "Examples:\n");
     fprintf(stderr, "   # Get the consensus for one region. The fasta header lines are then expected\n");
@@ -723,14 +845,17 @@ int main_consensus(int argc, char *argv[])
         {"output",1,0,'o'},
         {"fasta-ref",1,0,'f'},
         {"mask",1,0,'m'},
+        {"missing",1,0,'M'},
         {"chain",1,0,'c'},
+        {"prefix",required_argument,0,'p'},
         {0,0,0,0}
     };
     int c;
-    while ((c = getopt_long(argc, argv, "h?s:1Ii:e:H:f:o:m:c:",loptions,NULL)) >= 0) 
+    while ((c = getopt_long(argc, argv, "h?s:1Ii:e:H:f:o:m:c:M:p:",loptions,NULL)) >= 0) 
     {
         switch (c) 
         {
+            case 'p': args->chr_prefix = optarg; break;
             case 's': args->sample = optarg; break;
             case 'o': args->output_fname = optarg; break;
             case 'I': args->output_iupac = 1; break;
@@ -738,6 +863,10 @@ int main_consensus(int argc, char *argv[])
             case 'i': args->filter_str = optarg; args->filter_logic |= FLT_INCLUDE; break;
             case 'f': args->ref_fname = optarg; break;
             case 'm': args->mask_fname = optarg; break;
+            case 'M': 
+                args->missing_allele = optarg[0]; 
+                if ( optarg[1]!=0 ) error("Expected single character with -M, got \"%s\"\n", optarg);
+                break;
             case 'c': args->chain_fname = optarg; break;
             case 'H': 
                 if ( !strcasecmp(optarg,"R") ) args->allele |= PICK_REF;
@@ -748,10 +877,14 @@ int main_consensus(int argc, char *argv[])
                 else if ( !strcasecmp(optarg,"LA") ) args->allele |= PICK_LONG|PICK_ALT;
                 else if ( !strcasecmp(optarg,"SR") ) args->allele |= PICK_SHORT|PICK_REF;
                 else if ( !strcasecmp(optarg,"SA") ) args->allele |= PICK_SHORT|PICK_ALT;
+                else if ( !strcasecmp(optarg,"1pIu") ) args->allele |= PICK_IUPAC, args->haplotype = 1;
+                else if ( !strcasecmp(optarg,"2pIu") ) args->allele |= PICK_IUPAC, args->haplotype = 2;
                 else
                 {
-                    args->haplotype = optarg[0] - '0'; 
-                    if ( args->haplotype <=0 ) error("Expected positive integer with --haplotype\n");
+                    char *tmp;
+                    args->haplotype = strtol(optarg, &tmp, 10);
+                    if ( tmp==optarg || *tmp ) error("Error: Could not parse --haplotype %s, expected numeric argument\n", optarg);
+                    if ( args->haplotype <=0 ) error("Error: Expected positive integer with --haplotype\n");
                 }
                 break;
             default: usage(args); break;
