@@ -195,7 +195,7 @@ int init(int argc, char **argv, bcf_hdr_t *in, bcf_hdr_t *out)
         {
             case 'i': args->filter_str = optarg; args->filter_logic = FLT_INCLUDE; break;
             case 'e': args->filter_str = optarg; args->filter_logic = FLT_EXCLUDE; break;
-            case 'n': args->new_mask = bcf_gt_phased(0); 
+            case 'n': args->new_mask = 0;
                 if ( strchr(optarg,'.') ) args->new_mask |= GT_MISSING;
                 if ( strchr(optarg,'0') ) args->new_mask |= GT_REF;
                 if ( strchr(optarg,'M') ) args->new_mask |= GT_MAJOR;
@@ -232,6 +232,19 @@ int init(int argc, char **argv, bcf_hdr_t *in, bcf_hdr_t *out)
     return 0;
 }
 
+static inline int phase_gt(int32_t *ptr, int ngts)
+{
+    int j, changed = 0;
+    for (j=0; j<ngts; j++)
+    {
+        if ( ptr[j]==bcf_int32_vector_end ) break;
+        if ( bcf_gt_is_phased(ptr[j]) ) continue;
+        ptr[j] = bcf_gt_phased(bcf_gt_allele(ptr[j]));    // add phasing; this may need a fix, I think the flag should be set for one allele only?
+        changed++;
+    }
+    return changed;
+}
+
 static inline int unphase_gt(int32_t *ptr, int ngts)
 {
     int j, changed = 0;
@@ -307,7 +320,7 @@ bcf1_t *process(bcf1_t *rec)
         hts_expand(int,rec->n_allele,args->marr,args->arr);
         int ret = bcf_calc_ac(args->in_hdr,rec,args->arr,BCF_UN_FMT);
         if ( ret<= 0 )
-            error("Could not calculate allele count at %s:%d\n", bcf_seqname(args->in_hdr,rec),rec->pos+1);
+            error("Could not calculate allele count at %s:%"PRId64"\n", bcf_seqname(args->in_hdr,rec),(int64_t) rec->pos+1);
 
         for(i=0; i < rec->n_allele; ++i)
         {
@@ -340,14 +353,16 @@ bcf1_t *process(bcf1_t *rec)
             int ia = bcf_gt_allele(ptr[0]); 
             int ib = bcf_gt_allele(ptr[1]); 
             if ( ia>=nbinom || ib>=nbinom ) 
-                error("The sample %s has incorrect number of %s fields at %s:%d\n",
-                        args->in_hdr->samples[i],args->binom_tag,bcf_seqname(args->in_hdr,rec),rec->pos+1);
+                error("The sample %s has incorrect number of %s fields at %s:%"PRId64"\n",
+                        args->in_hdr->samples[i],args->binom_tag,bcf_seqname(args->in_hdr,rec),(int64_t) rec->pos+1);
 
             double prob = calc_binom(args->iarr[i*nbinom+ia],args->iarr[i*nbinom+ib]);
             if ( !args->binom_cmp(prob,args->binom_val) ) continue;
 
             if ( args->new_mask&GT_UNPHASED )
                 changed += unphase_gt(ptr, ngts);
+            else if ( args->new_mask==GT_PHASED )
+                changed += phase_gt(ptr, ngts);
             else
                 changed += set_gt(ptr, ngts, args->new_gt);
         }
@@ -376,9 +391,11 @@ bcf1_t *process(bcf1_t *rec)
 
         for (i=0; i<rec->n_sample; i++)
         {
-            if ( !args->smpl_pass[i] ) continue;
+            if ( args->smpl_pass && !args->smpl_pass[i] ) continue;
             if ( args->new_mask&GT_UNPHASED )
                 changed += unphase_gt(args->gts + i*ngts, ngts);
+            else if ( args->new_mask==GT_PHASED )
+                changed += phase_gt(args->gts + i*ngts, ngts);
             else
                 changed += set_gt(args->gts + i*ngts, ngts, args->new_gt);
         }
@@ -405,6 +422,8 @@ bcf1_t *process(bcf1_t *rec)
 
             if ( args->new_mask&GT_UNPHASED )
                 changed += unphase_gt(ptr, ngts);
+            else if ( args->new_mask==GT_PHASED )
+                changed += phase_gt(ptr, ngts);
             else
                 changed += set_gt(ptr, ngts, args->new_gt);
         }
