@@ -1,4 +1,3 @@
-
 /*  vcfindex.c -- Index bgzip compressed VCF/BCF files for random access.
 
     Copyright (C) 2014-2016 Genome Research Ltd.
@@ -32,6 +31,8 @@ DEALINGS IN THE SOFTWARE.  */
 #include <sys/stat.h>
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
+#include <htslib/kstring.h>
+#include <htslib/bgzf.h>
 #include "bcftools.h"
 
 #define BCF_LIDX_SHIFT    14
@@ -43,24 +44,22 @@ static void usage(void)
     fprintf(stderr, "Usage:   bcftools index [options] <in.bcf>|<in.vcf.gz>\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Indexing options:\n");
-    fprintf(stderr, "    -c, --csi            generate CSI-format index for VCF/BCF files [default]\n");
-    fprintf(stderr, "    -f, --force          overwrite index if it already exists\n");
-    fprintf(stderr, "    -m, --min-shift INT  set minimal interval size for CSI indices to 2^INT [14]\n");
-    fprintf(stderr, "    -t, --tbi            generate TBI-format index for VCF files\n");
+    fprintf(stderr, "    -c, --csi                generate CSI-format index for VCF/BCF files [default]\n");
+    fprintf(stderr, "    -f, --force              overwrite index if it already exists\n");
+    fprintf(stderr, "    -m, --min-shift INT      set minimal interval size for CSI indices to 2^INT [14]\n");
+    fprintf(stderr, "    -o, --output-file FILE   optional output index file name\n");
+    fprintf(stderr, "    -t, --tbi                generate TBI-format index for VCF files\n");
+    fprintf(stderr, "        --threads INT        use multithreading with INT worker threads [0]\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Stats options:\n");
     fprintf(stderr, "    -n, --nrecords       print number of records based on existing index file\n");
-    fprintf(stderr, "    -s, --stats   print per contig stats based on existing index file\n");
+    fprintf(stderr, "    -s, --stats          print per contig stats based on existing index file\n");
     fprintf(stderr, "\n");
     exit(1);
 }
 
 int vcf_index_stats(char *fname, int stats)
 {
-    char *fn_out = NULL;
-    FILE *out;
-    out = fn_out ? fopen(fn_out, "w") : stdout;
-
     const char **seq;
     int i, nseq;
     tbx_t *tbx = NULL;
@@ -74,12 +73,12 @@ int vcf_index_stats(char *fname, int stats)
     if ( hts_get_format(fp)->format==vcf )
     {
         tbx = tbx_index_load(fname);
-        if ( !tbx ) { fprintf(stderr,"Could not load TBI index: %s\n", fname); return 1; }
+        if ( !tbx ) { fprintf(stderr,"Could not load index for VCF: %s\n", fname); return 1; }
     }
     else if ( hts_get_format(fp)->format==bcf )
     {
         idx = bcf_index_load(fname);
-        if ( !idx ) { fprintf(stderr,"Could not load CSI index: %s\n", fname); return 1; }
+        if ( !idx ) { fprintf(stderr,"Could not load index for BCF file: %s\n", fname); return 1; }
     }
     else
     {
@@ -97,7 +96,7 @@ int vcf_index_stats(char *fname, int stats)
         if (stats&2 || !records) continue;
         bcf_hrec_t *hrec = bcf_hdr_get_hrec(hdr, BCF_HL_CTG, "ID", seq[i], NULL);
         int hkey = hrec ? bcf_hrec_find_key(hrec, "length") : -1;
-        fprintf(out,"%s\t%s\t%" PRIu64 "\n", seq[i], hkey<0?".":hrec->vals[hkey], records);
+        printf("%s\t%s\t%" PRIu64 "\n", seq[i], hkey<0?".":hrec->vals[hkey], records);
     }
     if (!sum)
     {
@@ -106,15 +105,14 @@ int vcf_index_stats(char *fname, int stats)
         bcf1_t *rec = bcf_init1();
         if (bcf_read1(fp, hdr, rec) >= 0)
         {
-            fprintf(stderr,"%s index of %s does not contain any count metadata. Please re-index with a newer version of bcftools or tabix.\n", tbx ? "TBI" : "CSI", fname);
+            fprintf(stderr,"index of %s does not contain any count metadata. Please re-index with a newer version of bcftools or tabix.\n", fname);
             return 1;
         }
         bcf_destroy1(rec);
     }
-    if (stats&2) fprintf(out, "%" PRIu64 "\n", sum);
+    if (stats&2) printf("%" PRIu64 "\n", sum);
     free(seq);
-    fclose(out);
-    hts_close(fp);
+    if ( hts_close(fp)!=0 ) error("[%s] Error: close failed\n", __func__);
     bcf_hdr_destroy(hdr);
     if (tbx)
         tbx_destroy(tbx);
@@ -125,8 +123,9 @@ int vcf_index_stats(char *fname, int stats)
 
 int main_vcfindex(int argc, char *argv[])
 {
-    int c, force = 0, tbi = 0, stats = 0;
+    int c, force = 0, tbi = 0, stats = 0, n_threads = 0;
     int min_shift = BCF_LIDX_SHIFT;
+    char *outfn = NULL;
 
     static struct option loptions[] =
     {
@@ -136,27 +135,33 @@ int main_vcfindex(int argc, char *argv[])
         {"min-shift",required_argument,NULL,'m'},
         {"stats",no_argument,NULL,'s'},
         {"nrecords",no_argument,NULL,'n'},
+        {"threads",required_argument,NULL,9},
+        {"output-file",required_argument,NULL,'o'},
         {NULL, 0, NULL, 0}
     };
 
     char *tmp;
-    while ((c = getopt_long(argc, argv, "ctfm:sn", loptions, NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "ctfm:sno:", loptions, NULL)) >= 0)
     {
         switch (c)
         {
             case 'c': tbi = 0; break;
             case 't': tbi = 1; min_shift = 0; break;
             case 'f': force = 1; break;
-            case 'm': 
+            case 'm':
                 min_shift = strtol(optarg,&tmp,10);
                 if ( *tmp ) error("Could not parse argument: --min-shift %s\n", optarg);
                 break;
             case 's': stats |= 1; break;
             case 'n': stats |= 2; break;
+            case 9:
+                n_threads = strtol(optarg,&tmp,10);
+                if ( *tmp ) error("Could not parse argument: --threads %s\n", optarg);
+                break;
+            case 'o': outfn = optarg; break;
             default: usage();
         }
     }
-    if ( optind==argc ) usage();
     if (stats>2)
     {
         fprintf(stderr, "[E::%s] expected only one of --stats or --nrecords options\n", __func__);
@@ -173,69 +178,55 @@ int main_vcfindex(int argc, char *argv[])
         return 1;
     }
 
-    char *fname = argv[optind];
+    char *fname = NULL;
+    if ( optind>=argc )
+    {
+        if ( !isatty(fileno((FILE *)stdin)) ) fname = "-";  // reading from stdin
+        else usage();
+    }
+    else fname = argv[optind];
     if (stats) return vcf_index_stats(fname, stats);
 
-    htsFile *fp = hts_open(fname,"r"); 
-    if ( !fp ) error("Failed to read %s\n", fname);
-    htsFormat type = *hts_get_format(fp);
-    hts_close(fp);
-
-    if ( (type.format!=bcf && type.format!=vcf) || type.compression!=bgzf )
+    kstring_t idx_fname = {0,0,0};
+    if (outfn)
+        kputs(outfn,&idx_fname);
+    else
     {
-        fprintf(stderr, "[E::%s] unknown filetype; expected bgzip compressed VCF or BCF\n", __func__);
-        if ( type.compression!=bgzf )
-            fprintf(stderr, "[E::%s] was the VCF/BCF compressed with bgzip?\n", __func__);
-        return 1;
+        if (!strcmp(fname, "-")) { fprintf(stderr, "[E::%s] must specify an output path for index file when reading VCF/BCF from stdin\n", __func__); return 1; }
+        ksprintf(&idx_fname, "%s.%s", fname, tbi ? "tbi" : "csi");
     }
-    if (tbi && type.format==bcf)
-    {
-        fprintf(stderr, "[Warning] TBI-index does not work for BCF files. Generating CSI instead.\n");
-        tbi = 0; min_shift = BCF_LIDX_SHIFT;
-    }
-    if (min_shift == 0 && type.format==bcf)
-    {
-        fprintf(stderr, "[E::%s] Require min_shift>0 for BCF files.\n", __func__);
-        return 1;
-    }
-    if (!tbi && type.format==vcf && min_shift == 0)
-    {
-        fprintf(stderr, "[Warning] min-shift set to 0 for VCF file. Generating TBI file.\n");
-        tbi = 1;
-    }
-
     if (!force)
     {
         // Before complaining about existing index, check if the VCF file isn't newer.
-        char *idx_fname = (char*)alloca(strlen(fname) + 5);
-        strcat(strcpy(idx_fname, fname), tbi ? ".tbi" : ".csi");
         struct stat stat_tbi, stat_file;
-        if ( stat(idx_fname, &stat_tbi)==0 )
+        if ( stat(idx_fname.s, &stat_tbi)==0 )
         {
             stat(fname, &stat_file);
             if ( stat_file.st_mtime <= stat_tbi.st_mtime )
             {
-                fprintf(stderr,"[E::%s] the index file exists. Please use '-f' to overwrite.\n", __func__);
+                fprintf(stderr,"[E::%s] the index file exists. Please use '-f' to overwrite %s\n", __func__, idx_fname.s);
+                free(idx_fname.s);
                 return 1;
             }
         }
+
+        // check for truncated files, allow only with -f
+        BGZF *fp = bgzf_open(fname, "r");
+        if ( !fp ) error("index: failed to open %s\n", fname);
+        if ( bgzf_compression(fp)!=2 ) error("index: the file is not BGZF compressed, cannot index: %s\n", fname);
+        if ( bgzf_check_EOF(fp)!=1 ) error("index: the input is probably truncated, use -f to index anyway: %s\n", fname);
+        if ( bgzf_close(fp)!=0 ) error("index: close failed: %s\n", fname);
     }
 
-    if (type.format==bcf)
-    {
-        if ( bcf_index_build(fname, min_shift) != 0 )
-        {
-            fprintf(stderr,"[E::%s] bcf_index_build failed for %s\n", __func__, fname);
-            return 1;
-        }
-    }
-    else
-    {
-        if ( tbx_index_build(fname, min_shift, &tbx_conf_vcf) != 0 )
-        {
-            fprintf(stderr,"[E::%s] tbx_index_build failed for %s\n", __func__, fname);
-            return 1;
-        }
+    int ret = bcf_index_build3(fname, idx_fname.s, min_shift, n_threads);
+    free(idx_fname.s);
+    if (ret != 0) {
+        if (ret == -2)
+            error("index: failed to open \"%s\"\n", fname);
+        else if (ret == -3)
+            error("index: \"%s\" is in a format that cannot be usefully indexed\n", fname);
+        else
+            error("index: failed to create index for \"%s\"\n", fname);
     }
     return 0;
 }
