@@ -1,6 +1,6 @@
 /*  vcfisec.c -- Create intersections, unions and complements of VCF files.
 
-    Copyright (C) 2012-2014 Genome Research Ltd.
+    Copyright (C) 2012-2019 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -33,6 +33,7 @@ THE SOFTWARE.  */
 #include <htslib/vcf.h>
 #include <htslib/synced_bcf_reader.h>
 #include <htslib/vcfutils.h>
+#include <htslib/hts_os.h>
 #include "bcftools.h"
 #include "filter.h"
 
@@ -41,6 +42,7 @@ THE SOFTWARE.  */
 #define OP_EQUAL 3
 #define OP_VENN 4
 #define OP_COMPLEMENT 5
+#define OP_EXACT 6
 
 // Logic of the filters: include or exclude sites which match the filters?
 #define FLT_INCLUDE 1
@@ -48,7 +50,7 @@ THE SOFTWARE.  */
 
 typedef struct
 {
-    int isec_op, isec_n, *write, iwrite, nwrite, output_type;
+    int isec_op, isec_n, *write, iwrite, nwrite, output_type, n_threads;
     int nflt, *flt_logic;
     filter_t **flt;
     char **flt_expr;
@@ -56,7 +58,8 @@ typedef struct
     FILE *fh_log, *fh_sites;
     htsFile **fh_out;
     char **argv, *prefix, *output_fname, **fnames, *write_files, *targets_list, *regions_list;
-    int argc;
+    char *isec_exact;
+    int argc, record_cmd_line;
 }
 args_t;
 
@@ -80,13 +83,13 @@ void mkdir_p(const char *fmt, ...)
     while (*p)
     {
         while (*p && *p!='/') p++;
-        if ( *p )
-        {
-            *p = 0;
-            mkdir(tmp,S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
-            *p = '/';
-            p++;
-        }
+        if ( !*p ) break;
+        char ctmp = *p;
+        *p = 0;
+        int ret = mkdir(tmp,S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
+        if ( ret!=0 && errno!=EEXIST ) error("Error creating directory %s: %s\n", path,strerror(errno));
+        *p = ctmp;
+        while ( *p && *p=='/' ) p++;
     }
     free(tmp);
     free(path);
@@ -126,14 +129,6 @@ FILE *open_file(char **fname, const char *mode, const char *fmt, ...)
     return fp;
 }
 
-const char *hts_bcf_wmode(int file_type)
-{
-    if ( file_type == FT_BCF ) return "wbu";    // uncompressed BCF
-    if ( file_type & FT_BCF ) return "wb";      // compressed BCF
-    if ( file_type & FT_GZ ) return "wz";       // compressed VCF
-    return "w";                                 // uncompressed VCF
-}
-
 void isec_vcf(args_t *args)
 {
     bcf_srs_t *files = args->files;
@@ -148,8 +143,9 @@ void isec_vcf(args_t *args)
     {
         out_fh = hts_open(args->output_fname? args->output_fname : "-",hts_bcf_wmode(args->output_type));
         if ( out_fh == NULL ) error("Can't write to %s: %s\n", args->output_fname? args->output_fname : "standard output", strerror(errno));
-        bcf_hdr_append_version(files->readers[args->iwrite].header,args->argc,args->argv,"bcftools_isec");
-        bcf_hdr_write(out_fh, files->readers[args->iwrite].header);
+        if ( args->n_threads ) hts_set_threads(out_fh, args->n_threads);
+        if (args->record_cmd_line) bcf_hdr_append_version(files->readers[args->iwrite].header,args->argc,args->argv,"bcftools_isec");
+        if ( bcf_hdr_write(out_fh, files->readers[args->iwrite].header)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->output_fname?args->output_fname:"standard output");
     }
     if ( !args->nwrite && !out_std && !args->prefix )
         fprintf(stderr,"Note: -w option not given, printing list of sites...\n");
@@ -190,13 +186,18 @@ void isec_vcf(args_t *args)
             case OP_COMPLEMENT: if ( n!=1 || !bcf_sr_has_line(files,0) ) continue; break;
             case OP_EQUAL: if ( n != args->isec_n ) continue; break;
             case OP_PLUS: if ( n < args->isec_n ) continue; break;
-            case OP_MINUS: if ( n > args->isec_n ) continue;
+            case OP_MINUS: if ( n > args->isec_n ) continue; break;
+            case OP_EXACT:
+                for (i=0; i<files->nreaders; i++)
+                    if ( files->has_line[i] != args->isec_exact[i] ) break;
+                if ( i<files->nreaders ) continue;
+                break;
         }
 
         if ( out_std )
         {
-            if ( bcf_sr_has_line(files,args->iwrite) )
-                bcf_write1(out_fh, files->readers[args->iwrite].header, files->readers[args->iwrite].buffer[0]);
+            if ( bcf_sr_has_line(files,args->iwrite) && bcf_write1(out_fh, files->readers[args->iwrite].header, files->readers[args->iwrite].buffer[0])!=0 )
+                error("[%s] Error: cannot write to %s\n", __func__, args->output_fname ? args->output_fname : "standard output");
             continue;
         }
         else if ( args->fh_sites )
@@ -218,7 +219,8 @@ void isec_vcf(args_t *args)
             for (i=0; i<files->nreaders; i++)
                 kputc(bcf_sr_has_line(files,i)?'1':'0', &str);
             kputc('\n', &str);
-            fwrite(str.s,sizeof(char),str.l,args->fh_sites);
+            if ( fwrite(str.s,sizeof(char),str.l,args->fh_sites)!=str.l )
+                error("[%s] Error: failed to write %d bytes to %s\n", __func__,(int)str.l,args->output_fname ? args->output_fname : "standard output");
         }
 
         if ( args->prefix )
@@ -226,9 +228,15 @@ void isec_vcf(args_t *args)
             if ( args->isec_op==OP_VENN && ret==3 )
             {
                 if ( !args->nwrite || args->write[0] )
-                    bcf_write1(args->fh_out[2], bcf_sr_get_header(files,0), bcf_sr_get_line(files,0));
+                {
+                    if ( bcf_write1(args->fh_out[2], bcf_sr_get_header(files,0), bcf_sr_get_line(files,0))!=0 )
+                         error("[%s] Error: cannot write\n", __func__);
+                }
                 if ( !args->nwrite || args->write[1] )
-                    bcf_write1(args->fh_out[3], bcf_sr_get_header(files,1), bcf_sr_get_line(files,1));
+                {
+                    if ( bcf_write1(args->fh_out[3], bcf_sr_get_header(files,1), bcf_sr_get_line(files,1))!=0 )
+                        error("[%s] Error: cannot write\n", __func__);
+                }
             }
             else
             {
@@ -236,13 +244,13 @@ void isec_vcf(args_t *args)
                 {
                     if ( !bcf_sr_has_line(files,i) ) continue;
                     if ( args->write && !args->write[i] ) continue;
-                    bcf_write1(args->fh_out[i], files->readers[i].header, files->readers[i].buffer[0]);
+                    if ( bcf_write1(args->fh_out[i], files->readers[i].header, files->readers[i].buffer[0])!=0 ) error("[%s] Error: cannot write\n", __func__);
                 }
             }
         }
     }
     if ( str.s ) free(str.s);
-    if ( out_fh ) hts_close(out_fh);
+    if ( out_fh && hts_close(out_fh)!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->output_fname? args->output_fname : "-");
 }
 
 static void add_filter(args_t *args, char *expr, int logic)
@@ -294,6 +302,16 @@ static void init_data(args_t *args)
         }
     }
 
+    if ( args->isec_op==OP_EXACT )
+    {
+        if ( strlen(args->isec_exact)!=args->files->nreaders )
+            error("The number of files does not match the bitmask: %d vs %s\n", args->files->nreaders,args->isec_exact);
+        for (i=0; i<args->files->nreaders; i++)
+            if ( args->isec_exact[i]!='0' && args->isec_exact[i]!='1' ) error("Unexpected bitmask: %s\n",args->isec_exact);
+        for (i=0; i<args->files->nreaders; i++)
+            args->isec_exact[i] -= '0';
+    }
+
     // Which files to write: parse the string passed with -w
     char *p = args->write_files;
     while (p && *p)
@@ -307,7 +325,7 @@ static void init_data(args_t *args)
         while (*p && *p!=',') p++;
         if ( *p==',' ) p++;
     }
-    if ( args->nwrite>1 && !args->prefix ) error("Expected -p when mutliple output files given: --write %s\n", args->write_files);
+    if ( args->nwrite>1 && !args->prefix ) error("Expected -p when multiple output files given: --write %s\n", args->write_files);
     if ( args->isec_op==OP_COMPLEMENT && args->nwrite )
     {
         if ( args->nwrite>1 ) error("Multiple files to -w make no sense with -C\n");
@@ -340,8 +358,9 @@ static void init_data(args_t *args)
                 open_file(&args->fnames[i], NULL, "%s/%04d.%s", args->prefix, i, suffix); \
                 args->fh_out[i] = hts_open(args->fnames[i], hts_bcf_wmode(args->output_type));  \
                 if ( !args->fh_out[i] ) error("Could not open %s\n", args->fnames[i]); \
-                bcf_hdr_append_version(args->files->readers[j].header,args->argc,args->argv,"bcftools_isec"); \
-                bcf_hdr_write(args->fh_out[i], args->files->readers[j].header); \
+                if ( args->n_threads ) hts_set_threads(args->fh_out[i], args->n_threads); \
+                if (args->record_cmd_line) bcf_hdr_append_version(args->files->readers[j].header,args->argc,args->argv,"bcftools_isec"); \
+                if ( bcf_hdr_write(args->fh_out[i], args->files->readers[j].header)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->fnames[i]); \
             }
             if ( !args->nwrite || args->write[0] )
             {
@@ -414,7 +433,7 @@ static void destroy_data(args_t *args)
         for (i=0; i<n; i++)
         {
             if ( !args->fnames[i] ) continue;
-            hts_close(args->fh_out[i]);
+            if ( hts_close(args->fh_out[i])!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->fnames[i]);
             if ( args->output_type==FT_VCF_GZ )
             {
                 tbx_conf_t conf = tbx_conf_vcf;
@@ -445,7 +464,8 @@ static void usage(void)
     fprintf(stderr, "    -e, --exclude <expr>          exclude sites for which the expression is true\n");
     fprintf(stderr, "    -f, --apply-filters <list>    require at least one of the listed FILTER strings (e.g. \"PASS,.\")\n");
     fprintf(stderr, "    -i, --include <expr>          include only sites for which the expression is true\n");
-    fprintf(stderr, "    -n, --nfiles [+-=]<int>       output positions present in this many (=), this many or more (+), or this many or fewer (-) files\n");
+    fprintf(stderr, "        --no-version                  do not append version and command line to the header\n");
+    fprintf(stderr, "    -n, --nfiles [+-=~]<int>      output positions present in this many (=), this many or more (+), this many or fewer (-), the exact (~) files\n");
     fprintf(stderr, "    -o, --output <file>           write output to a file [standard output]\n");
     fprintf(stderr, "    -O, --output-type <b|u|z|v>   b: compressed BCF, u: uncompressed BCF, z: compressed VCF, v: uncompressed VCF [v]\n");
     fprintf(stderr, "    -p, --prefix <dir>            if given, subset each of the input files accordingly, see also -w\n");
@@ -453,6 +473,7 @@ static void usage(void)
     fprintf(stderr, "    -R, --regions-file <file>     restrict to regions listed in a file\n");
     fprintf(stderr, "    -t, --targets <region>        similar to -r but streams rather than index-jumps\n");
     fprintf(stderr, "    -T, --targets-file <file>     similar to -R but streams rather than index-jumps\n");
+    fprintf(stderr, "        --threads <int>           use multithreading with <int> worker threads [0]\n");
     fprintf(stderr, "    -w, --write <list>            list of files to write with -p given as 1-based indexes. By default, all files are written\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Examples:\n");
@@ -465,6 +486,9 @@ static void usage(void)
     fprintf(stderr, "   # Extract and write records from A shared by both A and B using exact allele match\n");
     fprintf(stderr, "   bcftools isec A.vcf.gz B.vcf.gz -p dir -n =2 -w 1\n");
     fprintf(stderr, "\n");
+    fprintf(stderr, "   # Extract and write records from C found in A and C but not in B\n");
+    fprintf(stderr, "   bcftools isec A.vcf.gz B.vcf.gz C.vcf.gz -p dir -n~101 -w 3\n");
+    fprintf(stderr, "\n");
     fprintf(stderr, "   # Extract records private to A or B comparing by position only\n");
     fprintf(stderr, "   bcftools isec A.vcf.gz B.vcf.gz -p dir -n -1 -c all\n");
     fprintf(stderr, "\n");
@@ -479,26 +503,30 @@ int main_vcfisec(int argc, char *argv[])
     args->argc   = argc; args->argv = argv;
     args->output_fname = NULL;
     args->output_type = FT_VCF;
+    args->n_threads = 0;
+    args->record_cmd_line = 1;
     int targets_is_file = 0, regions_is_file = 0;
 
     static struct option loptions[] =
     {
-        {"help",0,0,'h'},
-        {"exclude",1,0,'e'},
-        {"include",1,0,'i'},
-        {"collapse",1,0,'c'},
-        {"complement",0,0,'C'},
-        {"apply-filters",1,0,'f'},
-        {"nfiles",1,0,'n'},
-        {"prefix",1,0,'p'},
-        {"write",1,0,'w'},
-        {"targets",1,0,'t'},
-        {"targets-file",1,0,'T'},
-        {"regions",1,0,'r'},
-        {"regions-file",1,0,'R'},
-        {"output",1,0,'o'},
-        {"output-type",1,0,'O'},
-        {0,0,0,0}
+        {"help",no_argument,NULL,'h'},
+        {"exclude",required_argument,NULL,'e'},
+        {"include",required_argument,NULL,'i'},
+        {"collapse",required_argument,NULL,'c'},
+        {"complement",no_argument,NULL,'C'},
+        {"apply-filters",required_argument,NULL,'f'},
+        {"nfiles",required_argument,NULL,'n'},
+        {"prefix",required_argument,NULL,'p'},
+        {"write",required_argument,NULL,'w'},
+        {"targets",required_argument,NULL,'t'},
+        {"targets-file",required_argument,NULL,'T'},
+        {"regions",required_argument,NULL,'r'},
+        {"regions-file",required_argument,NULL,'R'},
+        {"output",required_argument,NULL,'o'},
+        {"output-type",required_argument,NULL,'O'},
+        {"threads",required_argument,NULL,9},
+        {"no-version",no_argument,NULL,8},
+        {NULL,0,NULL,0}
     };
     while ((c = getopt_long(argc, argv, "hc:r:R:p:n:w:t:T:Cf:o:O:i:e:",loptions,NULL)) >= 0) {
         switch (c) {
@@ -523,7 +551,9 @@ int main_vcfisec(int argc, char *argv[])
                 else error("The --collapse string \"%s\" not recognised.\n", optarg);
                 break;
             case 'f': args->files->apply_filters = optarg; break;
-            case 'C': args->isec_op = OP_COMPLEMENT; break;
+            case 'C':
+                if ( args->isec_op!=0 && args->isec_op!=OP_COMPLEMENT ) error("Error: either -C or -n should be given, not both.\n");
+                args->isec_op = OP_COMPLEMENT; break;
             case 'r': args->regions_list = optarg; break;
             case 'R': args->regions_list = optarg; regions_is_file = 1; break;
             case 't': args->targets_list = optarg; break;
@@ -534,17 +564,23 @@ int main_vcfisec(int argc, char *argv[])
             case 'e': add_filter(args, optarg, FLT_EXCLUDE); break;
             case 'n':
                 {
+                    if ( args->isec_op!=0 && args->isec_op==OP_COMPLEMENT ) error("Error: either -C or -n should be given, not both.\n");
+                    if ( args->isec_op!=0 ) error("Error: -n should be given only once.\n");
                     char *p = optarg;
                     if ( *p=='-' ) { args->isec_op = OP_MINUS; p++; }
                     else if ( *p=='+' ) { args->isec_op = OP_PLUS; p++; }
                     else if ( *p=='=' ) { args->isec_op = OP_EQUAL; p++; }
+                    else if ( *p=='~' ) { args->isec_op = OP_EXACT; p++; }
                     else if ( isdigit(*p) ) args->isec_op = OP_EQUAL;
                     else error("Could not parse --nfiles %s\n", optarg);
-                    if ( sscanf(p,"%d",&args->isec_n)!=1 ) error("Could not parse --nfiles %s\n", optarg);
+                    if ( args->isec_op == OP_EXACT ) args->isec_exact = p;
+                    else if ( sscanf(p,"%d",&args->isec_n)!=1 ) error("Could not parse --nfiles %s\n", optarg);
                 }
                 break;
+            case  9 : args->n_threads = strtol(optarg, 0, 0); break;
+            case  8 : args->record_cmd_line = 0; break;
             case 'h':
-            case '?': usage();
+            case '?': usage(); break;
             default: error("Unknown argument: %s\n", optarg);
         }
     }
@@ -561,7 +597,7 @@ int main_vcfisec(int argc, char *argv[])
     if ( !args->targets_list )
     {
         if ( argc-optind<2  ) error("Expected multiple files or the --targets option\n");
-        if ( !args->isec_op ) error("Expected two file names or one of the options --complement, --nfiles or --targets\n");
+        if ( !args->isec_op ) error("One of the options --complement, --nfiles or --targets must be given with more than two files\n");
     }
     args->files->require_index = 1;
     while (optind<argc)
